import os
from datetime import datetime
import shutil
import numpy as np
from matplotlib import pyplot as plt
from pandas import DataFrame
import matplotlib.lines as mlines
import pandas

#expids = ['a327','a328','a329','a33u','a33v']
expids = ['a27e','a27f','a27g','a27m','a274']

#### create new directory
os.chdir(os.getcwd()+'/logs')
pwd = os.getcwd() + '/'
dt = datetime.now()
d = str(dt.year)[-2:]+'-'+"{:02d}".format(dt.month)+'-'+"{:02d}".format(dt.day)
procs = [int(i[-1]) for i in os.listdir(pwd) if d in i ]
procid = 0
print(procs)
if len(procs) > 0:
    while procid in procs:
        procid = procid+1
        print('in here')
procid=str(procid)
os.mkdir(pwd + d + '_' + procid)
os.chdir(pwd + d + '_' + procid)
pwd = os.getcwd() + '/'

### copy and edit copy_results.py

outputfile = 'PISCES_5d_19600101_19641231_ptrc_T.nc'

scr = '/gpfs/scratch/bsc32/bsc32786/'

for expid in expids:
    os.chdir(pwd)
    if expid in os.listdir(os.getcwd()):
        shutil.rmtree(pwd + expid)
    os.mkdir(pwd + expid)
    os.chdir(pwd + expid)

    os.mkdir('parameters')
    os.chdir('parameters')
    fs = os.listdir(scr + expid + '/EXP00/parameters/')
    for fi in fs:
        shutil.copyfile(scr + expid + '/EXP00/parameters/' + fi, os.getcwd() + '/' + fi)
    os.remove('0000.csv')

    os.chdir(pwd + expid)
    os.mkdir('stats')
    os.chdir('stats')
    fs = os.listdir(scr + expid + '/EXP00/bPOC/')
    for fi in fs:
        shutil.copyfile(scr + expid + '/EXP00/bPOC/' + fi, os.getcwd() + '/' + fi)
    fs = os.listdir(scr + expid + '/EXP00/sPOC/')
    for fi in fs:
        shutil.copyfile(scr + expid + '/EXP00/sPOC/' + fi, os.getcwd() + '/' + fi)

    # obtain highest performing file

    os.chdir(pwd + expid)
    fs = sorted(os.listdir(pwd + expid + '/parameters'))
    paras = np.genfromtxt(pwd + expid + '/parameters/'+fs[-1])
    minind = np.where(paras[:, -1] == min(paras[:, -1]))[0][0]
    chunk = "{0:0=4d}".format(minind+1)
    shutil.copyfile(scr + expid + '/EXP00/'+ chunk + '/' + outputfile , os.getcwd() + '/' + outputfile + expid)

### copy and edit plot_parametersmulti.py

os.chdir('..')

def make_grid(n):
    y=1
    x=1
    while y*x < n:
        if y==x:
            y=y+1
        else:
            x=x+1
    j=0
    i=0
    tuples=[]
    for a in range(n):
        tuples.append((j,i))
        if i == x - 1:
            i=0
            j=j+1
        else:
            i=i+1

    return y,x,tuples

runs = expids

basepath=os.getcwd() + '/'
paraspath='/parameters/'
bbp700path='/bbp700/'
modelfile='PISCES_5d_19600101_19691231_ptrc_T.nc'
obsfile='/gpfs/scratch/bsc32/bsc32786/pisces1d/obs/Mprof_6901486_L75_5days_2015-01-01_2015-12-31reg.nc'

templatepath = '/gpfs/scratch/bsc32/bsc32786/pisces1d/parameters/variables5wchld.csv'
template = pandas.read_csv(templatepath)
parameters=list(template['var'])
defaults=list(template['defaults'])
mins=list(template['min'])
maxes=list(template['max'])
numparams = len(parameters)


Y,X,T = make_grid(numparams)
print(Y,X,T)

cols=['C0','C1','C2','C3','C4','C5']

fig_size = plt.rcParams["figure.figsize"]
fig_size[0] = 12.8
fig_size[1] = 9.6

plt.rcParams["figure.figsize"] = fig_size
plt.locator_params(nbins=6)

fig, axes = plt.subplots(Y, X, sharex=True)
fig.text(0.5, 0.04, 'Generation', ha='center', size='large')

print([ i for i in dir(axes) if 'x' in i ])

costs={}

for r in range(len(runs)):
    costs[runs[r]]=[]
    stats = []
    for a in range(numparams):
        stats.append([])
    parafilelst = os.listdir(basepath + runs[r] + paraspath)
    parafilelst.sort()

    parasDF = DataFrame(index=range(len(parafilelst)), columns=parameters)

    for f in parafilelst:
        paras = np.genfromtxt(basepath + runs[r] + paraspath + f)
        minind = np.where(paras[:, -1] == min(paras[:, -1]))
        costs[runs[r]].append(float(paras[minind, -1][0][0]))
        for a in range(numparams):
            stats[a].append(paras[minind][0][a])

    for a in range(numparams):
        axes[T[a][0], T[a][1]].plot(range(len(stats[a])), stats[a], color=cols[r])#, color='grey', ls='--', linewidth=1)
        parasDF[parameters[a]]=stats[a]

    parasDF.to_csv('parameters_'+runs[r]+'.csv')

for a in range(numparams):
    axes[T[a][0],T[a][1]].plot( range(len(costs[runs[0]])),  [defaults[a]]*len(costs[runs[0]]), color='grey', ls='--', linewidth=1)
    axes[T[a][0],T[a][1]].plot( range(len(costs[runs[0]])),  [mins[a]]*len(costs[runs[0]]), color='grey', ls=':', linewidth=1)
    axes[T[a][0],T[a][1]].plot( range(len(costs[runs[0]])),  [maxes[a]]*len(costs[runs[0]]), color='grey', ls=':', linewidth=1)
    axes[T[a][0], T[a][1]].set_title(parameters[a])

legs=[]
cols=['C0','C1','C2','C3','C4','C5']
legs.append(mlines.Line2D([], [], color='gray', ls='--', label='Default'))
for r in range(len(runs)):
    legs.append(mlines.Line2D([], [], color=cols[r], ls='-', label=runs[r]))
leg1=fig.legend(handles=legs, loc='center right', bbox_to_anchor=(1, 0.5))
plt.gca().add_artist(leg1)

for j in range(len(axes[:,0])):
    for i in range(len(axes[0,:])):
        if not axes[j,i].lines:
            fig.delaxes(axes[j][i])

plt.savefig('parameters.png')
plt.cla()

fig_size = plt.rcParams["figure.figsize"]
fig_size[0] = 6.4
fig_size[1] = 4.8
plt.rcParams["figure.figsize"] = fig_size

fig = plt.figure()


costsDF=DataFrame.from_dict(costs,orient='index').transpose()
costsDF.to_csv('costs.csv')

for r in runs:
    plt.plot( range(len(costs[r])), costs[r])

legs=[]
cols=['C0','C1','C2','C3','C4','C5']
legs.append(mlines.Line2D([], [], color='gray', ls='--', label='Default'))
for r in range(len(runs)):
    legs.append(mlines.Line2D([], [], color=cols[r], ls='-', label=runs[r]))
leg1=fig.legend(handles=legs, loc='center right', bbox_to_anchor=(1, 0.5))
plt.gca().add_artist(leg1)
plt.xlabel('Generation')
plt.ylabel('Cost (ST Score)')
plt.savefig('costs.png')

statnames = ['sPOC', 'bPOC']
for statname in statnames:
    fig, axes = plt.subplots(2, 2, sharex=True)
    fig.text(0.5, 0.04, 'Generation', ha='center', size='medium')
    costs={}

    for r in range(len(runs)):

        filelst=sorted([i for i in os.listdir(basepath + runs[r] + '/stats/') if statname in i])

        allbias=[]
        allstd=[]
        allr=[]
        allrmse=[]
        for f in range(len(filelst)):
            data = pandas.read_csv(basepath + runs[r] + '/stats/' + filelst[f])
            #print(data)
            allbias.append(data['bias'][0])
            allstd.append(data['normstd'][0])
            allr.append(data['R'][0])
            allrmse.append(data['RMSE'][0])

        axes[0,0].plot(allbias, color=cols[r])
        axes[0, 1].plot(allstd, color=cols[r])
        axes[1, 0].plot(allr, color=cols[r])
        axes[1, 1].plot(allrmse, color=cols[r])

        statsDF = DataFrame(index=range(len(filelst)), columns=['bias','normstd','R','RMSE'])
        statsDF['bias']=allbias
        statsDF['normstd']=allstd
        statsDF['R']=allr
        statsDF['RMSE']=allrmse
        statsDF.to_csv(statname + '_stats_' + runs[r] + '.csv')

    legs=[]
    cols=['C0','C1','C2','C3','C4','C5']
    legs.append(mlines.Line2D([], [], color='gray', ls='--', label='Default'))
    for r in range(len(runs)):
        legs.append(mlines.Line2D([], [], color=cols[r], ls='-', label=runs[r]))
    leg1=fig.legend(handles=legs, loc='center right', bbox_to_anchor=(1, 0.5))
    plt.gca().add_artist(leg1)

    axes[ 0, 0].set_title('Bias',size='medium')
    axes[ 0, 1].set_title('Standard Deviation',size='medium')
    axes[ 1, 0].set_title('Correlation',size='medium')
    axes[ 1, 1].set_title('RMSE',size='medium')

    plt.savefig(statname+'_stats.png')