#!/bin/bash

# simulation part of the neural network. This is run by individual chunks in parallel with each other

### 1. Creates directories for each member if they havent already done so, i.e. in the first generation
### 2. Edits the namelists of each member using the corresponding row of the 00.csv parameters file
### 3. Runs the simulation
### 4. Uses the outputs, reference bias and observed data to collect the cost and save as a .txt

mode=observed

## ALL SHOULD BE THE SAME AS THOSE IN pre.sh
SCRHOME=/gpfs/scratch/bsc32/bsc32786/
MODEL=${SCRHOME}%EXPID%/EXP00/
MODELFILE=PISCES_5d_19600101_19641231_ptrc_T.nc
VARIABLES=/gpfs/scratch/bsc32/bsc32786/pisces1d/parameters/variables5wchld.csv
NEMO=/gpfs/projects/bsc32/models/pisces_off/gapoc/cfgs/mP1D/EXP00/

if [ $mode = 'observed' ]
then
    # path to the observed data. (PATH AND FILE) MUST BE ON THE SAME GRID AS THE MODEL
    OBS=${SCRHOME}pisces1d/obs/Mprof_6901486_L75_5days_2015-01-01_2015-12-31reg.nc # labrador sea orig
    #OBS=${SCRHOME}pisces1d/obs/Mprof_6901527_L75_5days_2014-01-01_2014-12-31reg.nc  # labrador sea new
    #OBS=${SCRHOME}pisces1d/obs/Mprof_6901516_L75_5days_2014-01-01_2014-12-31reg.nc  # iceland basin

elif [ $mode = 'default' ]
then
    OBS=${SCRHOME}%EXPID%/EXP00/default/$MODELFILE
fi

### 1. Creates directories for each member if they havent already done so, i.e. in the first generation

shopt -s nullglob

chunk=%CHUNK%
printf -v chunk "%04d" $chunk

cd ${MODEL}
if [ ! -d $MODEL$chunk ]; then
    # MAKE EXPID DIRECTORY AND COPY ALL THE CONTENT FROM EXP00 (.XML, EXECUTABLES, AND NAMELISTS)
    mkdir -p $chunk
else
    rm $chunk/*
fi
cd $chunk
cp -R ${NEMO}nemo .
cp -R ${NEMO}namelist* .
cp -R ${NEMO}*.xml .
cp -R ${MODEL}cropped/* .

### 2. Edits the namelists of each member using the corresponding row of the 0000.csv parameters file

NAMELIST=namelist_pisces_cfg

#cd ${MODEL}$chunk/

module load python

function changenls {

nls="$(python - $1 $2 $3 $4 << END
# inputs:
# 1. Parameters file (0000.csv)
# 2. Variables file (variables.csv)
# 3. Namelist (namelist_pisces_ref)
# 4. Chunk number

import os
import pandas as pd
import numpy as np

values=np.genfromtxt('$1')
names=pd.read_csv('$2')

chunk=int('$4')-1

with open('$3','r') as nl:
    with open('temp','w+') as t:
        lines=nl.readlines()
        lines = [i[:-1] for i in lines]
        for l in lines:
            for j in range(len(values[chunk])):
                print('j',j)
                lstart=l
                lcomm=''
                if '!' in l:
                    lstart=l[:l.find('!')] # so that comments that contain the variable name don't get edited
                    lcomm=l[l.find('!'):]# + '\n'
                if ' ' + names['var'][j]+' ' in lstart and "=" in lstart:
                    lstart = '   '+ names['var'][j] + '  =  ' + str(values[chunk][j]) + ' '
                    break
            t.write(lstart + lcomm + '\n')

os.remove('$3')
os.rename('temp','$3')

END
)"
}

changenls ${MODEL}parameters/0000.csv $VARIABLES ${MODEL}$chunk/$NAMELIST %CHUNK%


### 3. Runs the simulation

cd ${MODEL}$chunk

module purge
module load intel/2017.4
module load impi/2018.4
module load netcdf/4.2
module load hdf5/1.8.19
module load perl

mpirun -np 1 ./nemo

### 4. Uses the outputs, reference bias and observed data to collect the cost and save as a .txt

module load netcdf/4.4.1.1
module load mkl
module load python

function getcost {

cost="$(python - $1 $2 $3 $4 $5 << END
# $1 - path to default data $2 - path to observed data $3 - Path to output

mode = '$5'

from netCDF4 import Dataset
import numpy as np
import os
import math

def editmoddata(arr,d,s, mn=None, std=None):
    arr=arr[:,:np.shape(d)[0]]
    arr[-s:]
    arr[arr == 200000039192547557376] = np.nan
    arr=np.log10(arr)
    if mn:
        return arr
    else:
        mn = np.nanmean(arr)
        std = np.nanstd(arr)
        return arr, mn, std

def getobsdata(A):
    A[A == 0.0] = np.nan
    A=np.log10(A)
    mn = np.nanmean(A)
    std = np.nanstd(A)
    return A,mn,std

if mode == 'observed':
    obs=Dataset('$2')
    sPOCobs = np.array(obs.variables['spoc'])
    sPOCobs=sPOCobs[:,:46]
    sPOCobs, sPOCobsmean, sPOCobsstd = getobsdata(sPOCobs)
    bPOCobs = np.array(obs.variables['bpoc'])
    bPOCobs=bPOCobs[:,:46]
    bPOCobs, bPOCobsmean, bPOCobsstd = getobsdata(bPOCobs)
    depths=np.array(obs.variables['deptht'])
    S=np.shape(bPOCobs)[0]
    obs.close()

elif mode == 'default':
    obs=Dataset('$1')
    depths = np.array(obs.variables['deptht'])
    S=int(np.shape(np.array(obs.variables['POC']))[0]/5)
    print('S is',S)
    sPOCobs = (np.array(obs.variables['POC'])[-S:, :, 1, 1]+np.array(obs.variables['PHY'])[-S:, :, 1, 1]+np.array(obs.variables['ZOO'])[-S:, :, 1, 1])
    sPOCobs=sPOCobs[:,:46]
    sPOCobs, sPOCobsmean, sPOCobsstd = editmoddata(sPOCobs,depths,S)
    bPOCobs = (np.array(obs.variables['GOC'])[-S:, :, 1, 1]+np.array(obs.variables['PHY2'])[-S:, :, 1, 1]+np.array(obs.variables['ZOO2'])[-S:, :, 1, 1])
    bPOCobs=bPOCobs[:,:46]
    bPOCobs, bPOCobsmean, bPOCobsstd = editmoddata(bPOCobs,depths,S)

# get the model data
os.chdir('$3')
mod = Dataset('$4')
sPOCmod = (np.array(mod.variables['POC'])[-S:, :, 1, 1]+np.array(mod.variables['PHY'])[-S:, :, 1, 1]+np.array(mod.variables['ZOO'])[-S:, :, 1, 1])
sPOCmod=sPOCmod[:,:46]
sPOCmod=editmoddata(sPOCmod,depths,S, mn=sPOCobsmean, std=sPOCobsstd)
bPOCmod = (np.array(mod.variables['GOC'])[-S:, :, 1, 1]+np.array(mod.variables['PHY2'])[-S:, :, 1, 1]+np.array(mod.variables['ZOO2'])[-S:, :, 1, 1])
bPOCmod=bPOCmod[:,:46]
bPOCmod=editmoddata(bPOCmod,depths,S, mn=bPOCobsmean, std=bPOCobsstd)

if mode == 'observed':
    sPOCmod = sPOCmod[:,:np.shape(sPOCobs)[1]]
    bPOCmod = bPOCmod[:,:np.shape(bPOCobs)[1]]

#f = 'dyna_grid_T.nc'
#dynagrid=Dataset(f)
#mldkz5 = np.array(dynagrid.variables['mldkz5'])[:,1,1]
#mldkz5 = np.array([sum(mldkz5[5*i:5*i+5])/5 for i in range(int(len(mldkz5)/5))])
#
#for i in range(len(mldkz5)):
#    ds = np.where(depths<mldkz5[i])
#    sPOCmod[i, ds] = np.nan
#    sPOCobs[i, ds] = np.nan
#    bPOCmod[i, ds] = np.nan
#    bPOCobs[i, ds] = np.nan
#
#pardm=np.array(mod.variables['PARDM'])[-S:,:46,1,1]
#sPOCmod[pardm > 0.4] = np.nan
#sPOCobs[pardm > 0.4] = np.nan
#bPOCmod[pardm > 0.4] = np.nan
#bPOCobs[pardm > 0.4] = np.nan
#mod.close()

# calculating the statistical parameters

names=['bias','normstd','R','RMSE']#,'cost']

bias=np.nanmean(sPOCmod-sPOCobs)
mstd=np.nanstd(sPOCmod)/np.nanstd(sPOCobs)
R=float(np.ma.corrcoef(np.ma.masked_invalid(sPOCobs.flatten()), np.ma.masked_invalid(sPOCmod.flatten()))[0,1])
RMSE=math.sqrt(np.nansum((sPOCobs-sPOCmod)**2)/sPOCobs.size)
stats=[bias,mstd,R,RMSE]
for i in range(len(names)):
    np.savetxt('sPOC'+names[i]+'.csv',[stats[i]])
    
bias=np.nanmean(bPOCmod-bPOCobs)
mstd=np.nanstd(bPOCmod)/np.nanstd(bPOCobs)
R=float(np.ma.corrcoef(np.ma.masked_invalid(bPOCobs.flatten()), np.ma.masked_invalid(bPOCmod.flatten()))[0,1])
RMSE=math.sqrt(np.nansum((bPOCobs-bPOCmod)**2)/bPOCobs.size)
stats=[bias,mstd,R,RMSE]
for i in range(len(names)):
    np.savetxt('bPOC'+names[i]+'.csv',[stats[i]])    

END
)"
}

getcost ${MODEL}default/$MODELFILE $OBS $MODEL$chunk $MODELFILE $mode
